@@ -1,7 +1,12 @@
 
 ##  digest -- hash digest functions for R
 ##
-##  Copyright (C) 2003 - 2018  Dirk Eddelbuettel <edd@debian.org>
+##  Copyright (C) 2003 - 2019  Dirk Eddelbuettel <edd@debian.org>
+##  Copyright (C) 2009 - 2019  Henrik Bengtsson
+##  Copyright (C) 2012 - 2019  Hannes Muehleisen
+##  Copyright (C) 2014 - 2019  Jim Hester
+##  Copyright (C) 2019         Kendon Bell
+##  Copyright (C) 2019         Matthew de Queljoe
 ##
 ##  This file is part of digest.
 ##
@@ -18,7 +23,6 @@
 ##  You should have received a copy of the GNU General Public License
 ##  along with digest.  If not, see <http://www.gnu.org/licenses/>.
 
-
 digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
                                   "xxhash32", "xxhash64", "murmur32", "spookyhash"),
                    serialize=TRUE,
@@ -34,17 +38,6 @@ digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
     algo <- match.arg(algo)
     errormode <- match.arg(errormode)
 
-    .errorhandler <- function(txt, obj="", mode="stop") {
-        if (mode == "stop") {
-            stop(txt, obj, call.=FALSE)
-        } else if (mode == "warn") {	    # #nocov
-            warning(txt, obj, call.=FALSE)  # nocov
-            return(invisible(NA))           # nocov
-        } else {
-            return(invisible(NULL))         # nocov
-        }
-    }
-
     if (is.infinite(length)) {
         length <- -1               # internally we use -1 for infinite len
     }
@@ -75,19 +68,9 @@ digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
         ## we support raw vectors, so no mangling of 'object' is necessary
         ## regardless of R version
         ## skip="auto" - skips the serialization header [SU]
-        if (any(!is.na(pmatch(skip,"auto")))) {
-            if (ascii) {
-                ## HB 14 Mar 2007:
-                ## Exclude serialization header (non-data dependent bytes but R
-                ## version specific).  In ASCII, the header consists of for rows
-                ## ending with a newline ('\n').  We need to skip these.
-                ## The end of 4th row is *typically* within the first 18 bytes
-                skip <- which(object[1:30] == as.raw(10))[4] # nocov
-            } else {
-                skip <- 14
-            }
-            ## Was: skip <- if (ascii) 18 else 14
-        }
+        if (any(!is.na(pmatch(skip,"auto"))))
+            skip <- set_skip(object, ascii)
+
     } else if (!is.character(object) && !inherits(object,"raw") &&
                algo %in% non_streaming_algos) {
         return(.errorhandler(paste("Argument object must be of type character",		    # #nocov
@@ -104,36 +87,17 @@ digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
     ## HB 14 Mar 2007:  null op, only turned to char if alreadt char
     ##if (!inherits(object,"raw"))
     ##  object <- as.character(object)
-    algoint <- switch(algo,
-                      md5=1,
-                      sha1=2,
-                      crc32=3,
-                      sha256=4,
-                      sha512=5,
-                      xxhash32=6,
-                      xxhash64=7,
-                      murmur32=8,
-                      spookyhash=9)
+    algoint <- algo_int(algo)
     if (file) {
         algoint <- algoint+100
-        object <- path.expand(object)
-        if (!file.exists(object)) {
-            return(.errorhandler("The file does not exist: ", object, mode=errormode)) # nocov
-        }
-        if (!isTRUE(!file.info(object)$isdir)) {
-            return(.errorhandler("The specified pathname is not a file: ", # nocov
-                                 object, mode=errormode))                  # nocov
-        }
-        if (file.access(object, 4)) {
-            return(.errorhandler("The specified file is not readable: ",   # #nocov
-                                 object, mode=errormode))                  # #nocov
-        }
+        object <- enc2utf8(path.expand(object))
+        check_file(object, errormode)
     }
     ## if skip is auto (or any other text for that matter), we just turn it
     ## into 0 because auto should have been converted into a number earlier
     ## if it was valid [SU]
     if (is.character(skip)) skip <- 0
-    if(algo %in% non_streaming_algos){
+    if (algo %in% non_streaming_algos) {
         val <- .Call(digest_impl,
                      object,
                      as.integer(algoint),
@@ -154,3 +118,56 @@ digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
 
     return(val)
 }
+
+## utility functions used by digest() and  getVDigest() below
+
+.errorhandler <- function(txt, obj="", mode="stop") {
+    if (mode == "stop") {                                                                # nocov start
+        stop(txt, obj, call.=FALSE)
+    } else if (mode == "warn") {
+        warning(txt, obj, call.=FALSE)
+        return(invisible(NA))
+    } else {
+        return(invisible(NULL))                                                          # nocov end
+    }
+}
+
+algo_int <- function(algo)
+    switch(
+        algo,
+        md5 = 1,
+        sha1 = 2,
+        crc32 = 3,
+        sha256 = 4,
+        sha512 = 5,
+        xxhash32 = 6,
+        xxhash64 = 7,
+        murmur32 = 8,
+        spookyhash = 9
+    )
+
+## HB 14 Mar 2007:
+## Exclude serialization header (non-data dependent bytes but R
+## version specific).  In ASCII, the header consists of for rows
+## ending with a newline ('\n').  We need to skip these.
+## The end of 4th row is *typically* within the first 18 bytes
+set_skip <- function(object, ascii){
+    if (!ascii)
+        return(14)
+    ## Was: skip <- if (ascii) 18 else 14
+    which(object[1:30] == as.raw(10))[4]         					# nocov
+}
+
+check_file <- function(object, errormode){
+    if (!file.exists(object)) {
+        return(.errorhandler("The file does not exist: ", object, mode=errormode)) 	# nocov start
+    }
+    if (!isTRUE(!file.info(object)$isdir)) {
+        return(.errorhandler("The specified pathname is not a file: ",
+                             object, mode=errormode))
+    }
+    if (file.access(object, 4)) {
+        return(.errorhandler("The specified file is not readable: ",
+                             object, mode=errormode))                  			# #nocov end
+    }
+}
